/*
 * PM 2018
 * Solar Phone Charger
 * Daniel-Ilie Sirbu 333CB
 */

#include "lcd.h"

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdio.h>

#define ADC_MAX_CHANNELS 4  
#define ADC_AREF_VOLTAGE 5
#define ADC_INIT_CHANNEL 0

#define ORIZONTAL OCR1A
#define VERTICAL OCR1B

int timeOverflowCount = 0;
int sens = 0;

typedef struct
{
  const char *name;
  uint8_t    channel;
} adc_channel_t;

/* Valorile citite de la fiecare canal. */
volatile uint16_t ADC_value[ADC_MAX_CHANNELS];

const adc_channel_t ADC_channels[] = {
  // Senzor de lumina conectat la ADC0
  {"Light0", 0}, 
  // Senzor de lumina conectat la ADC1
  {"Light1", 1},
  // Senzor de lumina conectat la ADC2
  {"Light3", 2},
  // Senzor de lumina conectat la ADC3
  {"Light4", 3}

};

const int adc_num_channels = sizeof(ADC_channels) / sizeof(ADC_channels[0]);

ISR(ADC_vect)
{
    // Intreruperea este triggered cand o conversie este finalizata. */

    // * store new result
    // * change channel
    // * start a new conversion if there are any channels left
    static uint8_t channel = ADC_INIT_CHANNEL;

    ADC_value[channel] = ADC;

    // Canalele utilizate nu sunt consecutive!
    channel = (channel + 1) % ADC_MAX_CHANNELS; //4 canale de la 0 la 3

    ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

    // Pornim ADC-ul daca le-am citit pe toate.
    if (channel != ADC_INIT_CHANNEL)
        ADCSRA |= (1 << ADSC);
}

ISR(TIMER0_OVF_vect)
{
    // start an ADC conversion
    timeOverflowCount++;
    if (timeOverflowCount >= 6) { 
    	ADCSRA |= (1 << ADSC);
    	timeOverflowCount = 0;
    }
}

void ADC_init()
{
	ADMUX = 0;
	ADMUX |= (1 << REFS0) | (1 << ADC_INIT_CHANNEL);;
	ADCSRA = 0;
	ADCSRA |= (1 << ADEN) | (1 << ADIE) | (5 << ADPS0);
}

uint16_t ADC_get(uint8_t channel)
{
	return ADC_value[channel];
}

double ADC_voltage(int raw)
{
    (void)raw;
    return raw * ADC_AREF_VOLTAGE / 1023.0;
}
void ADC_show(void)
{

    LCD_printAt(0, "SRB Solar Phone");
    LCD_printAt(0xC0, "Charger  2018");
    const adc_channel_t *channel0 = &ADC_channels[0];
    double senzor0 = ADC_voltage(ADC_get(channel0->channel));
    
    const adc_channel_t *channel1 = &ADC_channels[1];
    double senzor1 = ADC_voltage(ADC_get(channel1->channel));

    const adc_channel_t *channel2 = &ADC_channels[2];
    double senzor2 = ADC_voltage(ADC_get(channel2->channel));

    const adc_channel_t *channel3 = &ADC_channels[3];
    double senzor3 = ADC_voltage(ADC_get(channel3->channel));

    while (1) {
        
        if (abs(senzor2 - senzor3) > 0.01) {
                if (senzor2 < senzor3) {
                    if (VERTICAL > 2000) {
                        VERTICAL -= 50;
                    }
                } else {
                    if (VERTICAL < 3300) {
                        VERTICAL += 50;
                    }
                }
                if (VERTICAL <= 2000 || VERTICAL >= 3300)
                    break;
            _delay_ms(50);

            senzor2 = ADC_voltage(ADC_get(channel2->channel));
            senzor3 = ADC_voltage(ADC_get(channel3->channel));
        } else
            break;
    }

    while (1) {
        if (VERTICAL < 2660)
            sens = 1;
        else
            sens = 2;
        if (abs(senzor0 - senzor1) > 0.01) {
                if (senzor0 < senzor1) {
                    if (sens == 1) {
                        if (ORIZONTAL < 4400)
                            ORIZONTAL += 50;
                    }
                    else if (sens == 2) 
                        if (ORIZONTAL > 1000)
                            ORIZONTAL -= 50;
                } else {
                    if (sens == 1){
                        if (ORIZONTAL > 1000)
                            ORIZONTAL -= 50;
                    }
                    else if (sens == 2) 
                        if (ORIZONTAL < 4400)
                            ORIZONTAL += 50;
                }
            _delay_ms(50);

            senzor0 = ADC_voltage(ADC_get(channel0->channel));
            senzor1 = ADC_voltage(ADC_get(channel1->channel));
        } else
            break;
    }

}
void timer0_init()
{
    TCCR0B = (5 << CS10);
    TIMSK0 |= (1 << TOIE1);
}

void timer1_init()
{
	DDRD |= _BV(PD5);
	DDRD |= _BV(PD4);
	TCCR1A |= 1 << WGM11 | 1 << COM1A1 | 1 << COM1B1;
	TCCR1B |= 1 << WGM13 | 1 << WGM12 | 1 << CS11;
	ICR1 = 39999;
    ORIZONTAL = 2600; //min = 1000
    VERTICAL = 2650; //mid = 2650 // max = 4500
}

void init()
{
	LcdDATA_DDR |= _BV(LcdA); 
    DDRD &= ~_BV(PD6); 
	PORTD |= _BV(PD6);
    DDRD &= ~_BV(PD0);
	PORTD |= _BV(PD0);

}

int main()
{

	init();
	timer1_init();

	DDRB |= (1 << PB0);
	PORTB |= (1 << PB0);

	DDRC |= _BV(PC0);
    PORTC |= _BV(PC0);
	LCD_DELAY();
	LCD_init();
	LCD_DELAY();
	
	ADC_init();
	timer0_init();
	sei();
    int apasat1 = 0;

    while(1) {
        LCD_A_LOW();
    	
    	ADC_show();

     	if(!(PIND & (1 << PD0))) { 
	    	if (!apasat1) {
                LCD_A_HIGH();
	    		_delay_ms(3000);
     			apasat1 = 1;
     		}
     	} else {
     		apasat1 = 0;
     	}
	}

    return 0;
}